#include <stdio.h>
#include <stdlib.h>

#include <string.h>

#include <assert.h>

#include <err.h>

#include <sys/queue.h>
#include <netdb.h>

#include <event.h>

#include "socket.h"
#include "nio.h"
#include "lookup.h"


static enum nio_action main_getln_catch(struct nio *io, const char *line, size_t len, enum nio_errno err, void *fd) {
	assert(err == NIO_ESUCCESS || err == NIO_ENOMORE);

	if (len > 0)
		warnx("read %d bytes: %.*s", (int)len, (int)(len - 1), line);

	if (err == NIO_ENOMORE) {
		nio_close(io);

		warnx("got EOF");

		return NIO_FINISH;
	}

	return NIO_RESTART;
} /* main_getln_catch() */


static enum nio_action main_write_catch(struct nio *io, const void *request, size_t len, enum nio_errno err, void *fd) {
	assert(err == NIO_ESUCCESS);

	if (len != strlen(request)) {
		assert(0 == nio_write(io, ((char *)request) + len, strlen(((char *)request) + len), &main_write_catch, io, &(struct timeval){ 2, 0 }));

		return 0;	
	}

	assert(0 == nio_getln(io, &main_getln_catch, io, &(struct timeval){ 2, 0 }));

	return 0;
} /* main_write_catch() */


static void main_connect_catch(int fd, struct sockaddr *sa, socklen_t salen, enum socket_errno err, void *arg) {
	static const char request[]	= "GET / HTTP/1.0\n\n";
	struct nio *io;

	assert(err == SOCKET_ESUCCESS);

	warnx("caught connect to google.com");

	assert(io = nio_open(fd, 0, 0, 0));

	socket_close(arg);

	assert(0 == nio_write(io, request, sizeof request, &main_write_catch, io, &(struct timeval){ 2, 0 }));

	return /* void */;
} /* main_connect_catch() */


int main(void) {
	struct socket_service *s;
	struct socket_name soname;

	event_init();
	lookup_init(0, 0, 0);

	if (0 == (s = socket_open(0, 0, 0)))
		err(EXIT_FAILURE, "socket_open");

	soname_set_host(&soname, "google.com", "http");

	socket_connect(0, s, &soname, &main_connect_catch, s, 0);

	event_loop(0);

	return 0;
}

