#include <stdio.h>
#include <stdlib.h>	/* EXIT_FAILURE */
#include <stdbool.h>	/* bool true false */
#include <inttypes.h>	/* intptr_t */

#include <signal.h>	/* SIGPIPE SIG_IGN signal(3) */

#include <errno.h>	/* EPIPE */

#include <string.h>	/* memset(3) */

#include <assert.h>	/* assert(3) */

#include <fcntl.h>	/* close(2) */

#include <sys/time.h>	/* struct timeval */
#include <sys/socket.h>	/* AF_INET socklen_t struct sockaddr
			 * struct sockaddr_storage */

#include <netinet/in.h>	/* struct sockaddr_in */

#include <unistd.h>	/* close(2) */

#include <err.h>	/* err(3) errx(3) */

#include <event.h>	/* struct event struct event_base event_init(3)
			 * event_base_loop(3) */

#include <arena/proto.h>

#include <socket.h>
#include <nio.h>


static const char mesg[]	= "one\ntwo\nthree\nbreak ";

static enum nio_action user_catch_write(struct nio *eio, const void *sent, size_t len, enum nio_errno err, void *fd) {
	static int nsent	= 0;

	assert(err == NIO_ESUCCESS);

	assert(len == sizeof mesg - 1);

	if (++nsent > 21) {
		nio_close(eio);

		assert(0 == close((intptr_t)fd));
	} else {
		assert(0 == nio_write(eio, mesg, sizeof mesg - 1, &user_catch_write, fd, &(struct timeval){ 2, 0 }));
	}

	return NIO_FINISH;
} /* user_catch_write() */


static enum nio_action serv_catch_line(struct nio *eio, const char *ln, size_t len, enum nio_errno err, void *arg) {
	struct socket_service *s	= arg;

	assert(err == NIO_ESUCCESS || err == NIO_ENOMORE);

	if (len > 0)
		warnx("read %d bytes: %.*s", (int)len, (int)(len - 1), ln);

	if (err == NIO_ENOMORE) {
		nio_close(eio);
		socket_close(s);

		return NIO_FINISH;
	} else {
		assert(len > 0);

		return NIO_RESTART;
	}
} /* serv_catch_line() */


static void main_catch_accept(int fd, struct sockaddr *sa, socklen_t salen, enum socket_errno err, void *arg) {
	struct nio *eio;

	if (err == SOCKET_ECANCELLED)
		return /* void */;

	assert(err == SOCKET_ESUCCESS);

	warnx("caught accept");

	assert(eio = nio_open(fd, 0, 0, 0));

	assert(0 == nio_getln(eio, &serv_catch_line, arg, &(struct timeval){ 2, 0} ));

	return /* void */;
} /* main_catch_accept() */


static void main_catch_connect(int fd, struct sockaddr *sa, socklen_t salen, enum socket_errno err, void *arg) {
	struct nio *eio;

	assert(err == SOCKET_ESUCCESS);

	warnx("caught connect");

	assert(eio = nio_open(fd, 0, 0, 0));

	assert(0 == nio_write(eio, mesg, sizeof mesg - 1, user_catch_write, (void *)(intptr_t)fd, &(struct timeval){ 2, 0 }));

	return /* void */;
} /* main_catch_connect() */


int main(void) {
	struct event_base *ev_base;
	struct socket_service *s;
	struct sockaddr_in sin;
	struct socket_options opts;
	struct socket_name soname;

	signal(SIGPIPE, SIG_IGN);

	if (!(ev_base = event_init()))
		err(EXIT_FAILURE, "event_init");

	(void)memset(&sin, '\0', sizeof sin);

	sin.sin_family		= AF_INET;
	sin.sin_port		= htons(4000);
	sin.sin_addr.s_addr	= htonl(INADDR_LOOPBACK);

	(void)soname_set_addr(&soname, (struct sockaddr *)&sin, sizeof sin);

	opts			= socket_defaults;
	opts.sa_reuseaddr	= 1;

	if (0 == (s = socket_open(&opts, ev_base, ARENA_STDLIB)))
		err(EXIT_FAILURE, "socket_open");

	if (0 != socket_accept(0, s, &soname, &main_catch_accept, s, 0))
		errx(EXIT_FAILURE, "socket_accept: %s", socket_strerror(s));

	socket_connect(0, s, &soname, &main_catch_connect, s, 0);

	event_base_loop(ev_base, 0);

	return 0;
} /* main() */

