#include <stdio.h>
#include <stdlib.h>	/* EXIT_FAILURE */

#include <string.h>	/* memset(3) */

#include <errno.h>

#include <fcntl.h>	/* close(2) */

#include <sys/time.h>	/* struct timeval */
#include <sys/socket.h>	/* AF_INET socklen_t struct sockaddr
			 * struct sockaddr_storage */

#include <netinet/in.h>	/* struct sockaddr_in */

#include <unistd.h>	/* close(2) */

#include <err.h>	/* err(3) errx(3) */

#include <event.h>	/* struct event struct event_base event_init(3)
			 * event_base_loop(3) */

#include <arena/proto.h>

#include <socket.h>


static void main_catch_accept(int fd, struct sockaddr *sa, socklen_t salen, enum socket_errno err, void *arg) {
	if (err == SOCKET_ECANCELLED)
		return /* void */;

	(void)close(fd);

	socket_close(arg);

	return /* void */;
} /* main_catch_accept() */


static void main_catch_connect(int fd, struct sockaddr *sa, socklen_t salen, enum socket_errno err, void *arg) {
	(void)close(fd);

	return /* void */;
} /* main_catch_connect() */


int main(void) {
	struct event_base *ev_base;
	struct socket_service *s;
	struct socket_name soname;
	struct sockaddr_in sin;
	struct socket_options opts;

	if (!(ev_base = event_init()))
		err(EXIT_FAILURE, "event_init");

	(void)memset(&sin, '\0', sizeof sin);

	sin.sin_family		= AF_INET;
	sin.sin_port		= htons(4000);
	sin.sin_addr.s_addr	= htonl(INADDR_LOOPBACK);

	(void)soname_set_addr(&soname, (struct sockaddr *)&sin, sizeof sin);

	opts			= socket_defaults;
	opts.sa_reuseaddr	= 1;

	if (0 == (s = socket_open(&opts, ev_base, ARENA_STDLIB)))
		err(EXIT_FAILURE, "socket_open");

	if (0 != socket_accept(0, s, &soname, &main_catch_accept, s, 0))
		errx(EXIT_FAILURE, "socket_accept: %s", socket_strerror(s));

	socket_connect(0, s, &soname, &main_catch_connect, 0, 0);

	event_base_loop(ev_base, 0);

	return 0;
} /* main() */

