#
# Standard GNU Make non-recursive prologue. Pushes callers $(d) onto a stack,
# for re-entrant Rules.mk processing.
#
sp		:= $(sp).x
dirstack_$(sp)	:= $(d)
d		:= $(dir)

# Our parent directory
$(d)/..		:= $(subst /$(notdir $(d)),,$(d))


#
# Make sure we have our defaults.
#
include $($(d)/..)/mk/Variables.mk


OBJS_$(d)	:= $(d)/nio.o $(d)/socket.o $(d)/thread.o $(d)/lookup.o
SRCS_$(d)	:= $(d)/nio.c $(d)/socket.c $(d)/thread.c $(d)/lookup.c
INCS_$(d)	:= $(d)/nio.h $(d)/socket.h $(d)/thread.h $(d)/lookup.h


CFLAGS_$(d)	:= $(CFLAGS)
CPPFLAGS_$(d)	:= -I$($(d)/..)/compat $(CPPFLAGS)



#
# Compilation Targets
#
$(d)/lookup.h: $(d)/lookup.h.in
	$(M4) $(M4FLAGS) -Dscope=private $(<D)/../m4/macros.m4 $< > $@

$(d)/%.o: $(d)/%.c $(INCS_$(d))
	$(CC) -c $(CPPFLAGS_$(@D)) $(CFLAGS_$(@D)) -o $@ $<

$(d)/libevnet.a: $(OBJS_$(d))
	$(AR) $(ARFLAGS) $@ $(OBJS_$(@D))
	$(RANLIB) $(RANLIBFLAGS) $@

all: $(d)/libevnet.a


#
# Install targets
#
$(DESTDIR)$(libdir)/libevnet.a: $(d)/libevnet.a | $(DESTDIR)$(libdir)
	$(INSTALL_DATA) $< $@

$(DESTDIR)$(includedir)/evnet: | $(DESTDIR)$(includedir)
	$(MKDIR) $@

$(DESTDIR)$(includedir)/evnet/nio.h: $(d)/nio.h | $(DESTDIR)$(includedir)/evnet
	$(INSTALL_DATA) $< $@

$(DESTDIR)$(includedir)/evnet/socket.h: $(d)/socket.h | $(DESTDIR)$(includedir)/evnet
	$(INSTALL_DATA) $< $@

$(DESTDIR)$(includedir)/evnet/thread.h: $(d)/thread.h | $(DESTDIR)$(includedir)/evnet
	$(INSTALL_DATA) $< $@

$(DESTDIR)$(includedir)/evnet/lookup.h: $(d)/lookup.h.in | $(DESTDIR)$(includedir)/evnet
	$(M4) $(M4FLAGS) -Dscope=public $(<D)/../m4/macros.m4 $< > $@

.INTERMEDIATE: -levnet

-levnet: $(DESTDIR)$(libdir)/libevnet.a $(DESTDIR)$(includedir)/evnet/nio.h \
	$(DESTDIR)$(includedir)/evnet/socket.h $(DESTDIR)$(includedir)/evnet/thread.h \
	$(DESTDIR)$(includedir)/evnet/lookup.h

install: -levnet


#
# Uninstall targets
#
.PHONY: $(d)/uninstall

$(d)/uninstall:
	rm -fr $(DESTDIR)$(includedir)/evnet
	rm -f $(DESTDIR)$(libdir)/libevnet.a

uninstall: $(d)/uninstall


#
# Clean targets
#
.PHONY: $(d)/clean

$(d)/clean:
	rm -f $(@D)/*.o $(@D)/*.a

clean: $(d)/clean


#
# Standard GNU Make non-recursive epilogue. Restores callers $(d), for
# re-entrant Rules.mk processing.
#
d	:= $(dirstack_$(sp))
sp	:= $(basename $(sp))

