/* ==========================================================================
 * libevnet/src/nio.h - Network server library for libevent.
 * --------------------------------------------------------------------------
 * Copyright (c) 2006  William Ahern
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to permit
 * persons to whom the Software is furnished to do so, subject to the
 * following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
 * NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
 * USE OR OTHER DEALINGS IN THE SOFTWARE.
 * ==========================================================================
 */
#ifndef EVNET_NIO_H
#define EVNET_NIO_H


/* Declare some things that the caller might not necessarily care about. */
struct event_base;	/* defined by libevent */
struct arena_prototype;	/* defined by libarena */


extern unsigned int nio_recurse_max;


enum nio_errno {
	NIO_ESUCCESS,	/* All's well. */
	NIO_ESYSTEM,	/* System error. See errno(2). */
	NIO_ETIMEDOUT,	/* Operation timed out. */
	NIO_EPENDING,	/* Conflicting operation already pending. */
	NIO_ECANCELLED,	/* Operation was explicitly cancelled. */
	NIO_ENOMORE,	/* No more data (aka end-of-file). */
	NIO_ENOSPACE,	/* Cannot restart read operation for lack of space. */
	NIO_ENODATA,	/* Cannot restart write operation for lack of data. */
}; /* enum nio_errno */

extern const char *nio_errlist[];

extern const int nio_nerr;


enum nio_mode {
	NIO_READ,
	NIO_WRITE,
	NIO_ABORT,
}; /* enum nio_mode */


enum nio_action {
	NIO_FINISH	= 0,
	NIO_RESTART	= 1,
}; /* enum nio_action */



struct nio;


extern const struct nio_options {
	unsigned int recurse_max;

	enum nio_mode (*sys_mode)(struct nio *, int fd, enum nio_mode, void *);

	ssize_t (*sys_read)(struct nio *, int fd, void *, size_t, void *);
	ssize_t (*sys_write)(struct nio *, int fd, const void *, size_t, void *);
} nio_defaults;


struct nio *nio_open(int, const struct nio_options *, struct event_base *, const struct arena_prototype *);

enum nio_errno nio_close(struct nio *);

typedef enum nio_action (*nio_read_cb)(struct nio *, void *, size_t, enum nio_errno, void *);

enum nio_errno nio_read(struct nio *, void *, size_t, nio_read_cb, void *, struct timeval *);

typedef enum nio_action (*nio_write_cb)(struct nio *, const void *, size_t, enum nio_errno, void *);

enum nio_errno nio_write(struct nio *, const void *, size_t, nio_write_cb, void *, struct timeval *);

typedef enum nio_action (*nio_getln_cb)(struct nio *, const char *, size_t, enum nio_errno, void *);

enum nio_errno nio_getln(struct nio *, nio_getln_cb, void *, struct timeval *);

int nio_sys_errno(struct nio *);

enum nio_errno nio_errno(struct nio *);

const char *nio_strerror(struct nio *);


#endif /* EVNET_NIO_H */
